@@ -58,7 +58,7 @@ SWSUSP_D="/etc/hibernate"
 SCRIPTLET_PATH="$SWSUSP_D/scriptlets.d /usr/local/share/hibernate/scriptlets.d /usr/share/hibernate/scriptlets.d"
 CONFIG_FILE="$SWSUSP_D/hibernate.conf"
 EXE=`basename $0`
-VERSION="1.12"
+VERSION="1.91"
 
 # Add these to the $PATH just in case.
 PATH="$PATH:/sbin:/usr/sbin"
@@ -71,6 +71,7 @@ vecho() {
     local v
     v="$1"
     shift
+    [ "x$LOG_TIMESTAMP" = "x1" ] && set -- $(date "+%b %e %H:%M:%S.%2N") "$@"
     if [ "$v" -le $VERBOSITY ] ; then
 	echo $@
     else
@@ -164,23 +165,11 @@ $WRAPPED_HELP
 }
 CMDLINE_OPTIONS_HELP=""
 
-# AddConfigHelp <item name> <item help>: Adds an option to the option help
-# text. <item help> must only contain line breaks if a new paragraph really
-# does want to be started. Text wrapping is taken care of.
+# AddConfigHelp <item name> <item help>: Deprecated. We no longer write
+# config help in --help
 AddConfigHelp() {
-    [ -n "$DISABLE_HELP" ] && return
-    local ATEXT
-    local WRAPPED_HELP
-    ATEXT="  $1"
-    [ -n "$CURRENT_SOURCED_SCRIPTLET" ] && ATEXT=`LeftRightPadText "$ATEXT" "[$CURRENT_SOURCED_SCRIPTLET]"`
-    WRAPPED_HELP="`echo \"$2\" | WrapHelpText`"
-    ATEXT="$ATEXT
-$WRAPPED_HELP
-
-"
-    CONFIGURATION_OPTIONS_HELP="$CONFIGURATION_OPTIONS_HELP$ATEXT"
+    return 0
 }
-CONFIGURATION_OPTIONS_HELP=""
 
 # FindXServer: Tries to find a running X server on the system. If one is found,
 # will set DISPLAY, XAUTHORITY and the XUSER variable, and return 0.
@@ -203,20 +192,38 @@ FindXServer() {
 
     # Find a useful XAUTHORITY and ideally a username too if we can!
     local xuser xauth xpid
-    for xpid in `pidof kwrapper ksmserver kdeinit gnome-session fvwm fvwm2 pwm blackbox fluxbox X XFree86 Xorg` ; do
-	# Ensure the process still exists, and we aren't hallucinating.
-	[ -d "/proc/$xpid/" ] || continue
-
-	xauth=`awk 'BEGIN{RS="\\000";FS="="}($1 == "XAUTHORITY"){print $2}' < /proc/$xpid/environ`
-	xhome=`awk 'BEGIN{RS="\\000";FS="="}($1 == "HOME"){print $2}' < /proc/$xpid/environ`
-	xuser=`/bin/ls -ld /proc/$xpid/ | awk '{print $3}'`
-	[ -z $xauth ] && [ -n $xhome ] && [ -f $xhome/.Xauthority ] && xauth=$xhome/.Xauthority
-
-	XAUTHORITY=$xauth su $xuser -c "$xhost" > /dev/null 2>&1 && break
-
-	xauth=
-	xuser=
+    for display in $(seq 0 8); do
+	xuser=$(last ":$display" | grep 'still logged in' | awk '{print $1}') || true
+	if [ -n "$xuser" ]; then
+	    xhome=$(getent passwd "$xuser" | cut -d: -f 6)
+	    if [ -n "$xhome" ]; then
+		xauth=$xhome/.Xauthority
+	    fi
+	fi
+	if [ -z "$xhome" ] || [ -z "$xauth" ]; then
+	    xauth=
+	    xuser=
+	    xhome=
+	else 
+	    break
+	fi
     done
+    if [ -z $xuser ] ; then
+        for xpid in `pidof kwrapper ksmserver kdeinit gnome-session fvwm fvwm2 pwm blackbox fluxbox X XFree86 Xorg` ; do
+	    # Ensure the process still exists, and we aren't hallucinating.
+	    [ -d "/proc/$xpid/" ] || continue
+
+	    xauth=`awk 'BEGIN{RS="\\000";FS="="}($1 == "XAUTHORITY"){print $2}' < /proc/$xpid/environ`
+	    xhome=`awk 'BEGIN{RS="\\000";FS="="}($1 == "HOME"){print $2}' < /proc/$xpid/environ`
+	    xuser=`/bin/ls -ld /proc/$xpid/ | awk '{print $3}'`
+	    [ -z $xauth ] && [ -n $xhome ] && [ -f $xhome/.Xauthority ] && xauth=$xhome/.Xauthority
+
+	    XAUTHORITY=$xauth su $xuser -c "$xhost" > /dev/null 2>&1 && break
+
+	   xauth=
+	   xuser=
+       done
+    fi
     if [ -n $xuser ] ; then
 	XUSER=$xuser
 	if [ -n "$xauth" ] ; then
@@ -382,10 +389,9 @@ Usage: $EXE [options]
 Activates software suspend and control its parameters.
 
 $CMDLINE_OPTIONS_HELP
-The following config file options are available (module name in brackets):
+For configuration file options for hibernate.conf, man hibernate.conf.
 
-$CONFIGURATION_OPTIONS_HELP
-Hibernate Script $VERSION                           (C) 2004 Bernard Blackham
+Hibernate Script $VERSION                      (C) 2004-2006 Bernard Blackham
 EOT
     return
 }
@@ -592,6 +598,13 @@ ProcessConfigOption() {
 	    [ -z "$LOGFILE" ] &&
 		LOGFILE="$params"
 	    ;;
+	logtimestamp)
+	    if BoolIsOn "$option" "$params" ; then
+		LOG_TIMESTAMP=1
+	    else
+		LOG_TIMESTAMP=0
+	    fi
+	    ;;
 	logverbosity)
 	    EnsureNumeric "$option" "$params" && LOG_VERBOSITY="$params"
 	    ;;
@@ -610,17 +623,30 @@ ProcessConfigOption() {
 	    export DISPLAY
 	    ;;
         include)
-            if [ -r "$params" ]; then
-                ReadConfigFile "$params"
-            else
+            if ! ReadConfigFile "$params" ; then
                 echo "$EXE: Unable to read configuration file $params (from Include directive)."
                 exit 1
             fi
             ;;
+	trymethod)
+	    if [ -z "$HIBERNATE_SUSPEND_METHOD" ] ; then
+		NO_COMPLAIN_UNSUPPORTED=1
+		vecho 1 "Trying method in $2..."
+		if ! ReadConfigFile "$params" ; then
+		    echo "$EXE: Unable to read configuration file $params (from TryMethod directive)."
+		fi
+		NO_COMPLAIN_UNSUPPORTED=
+	    fi
+	    ;;
 	*)
 	    if ! PluginConfigOption $option $params ; then
-		echo "$EXE: Unknown configuration option ($option)"
-		exit 1
+		# See if we're trying something new, and haven't yet got a suspend method
+		if [ -n "$NO_COMPLAIN_UNSUPPORTED" ] && [ -z "$HIBERNATE_SUSPEND_METHOD" ] ; then
+		    return 1
+		else
+		    echo "$EXE: Unknown configuration option ($option)"
+		    exit 1
+		fi
 	    fi
 	    ;;
     esac
@@ -633,9 +659,14 @@ ReadConfigFile() {
     local option params
     local file_name="$1"
     if [ ! -f "${file_name}" ] ; then
-	echo "WARNING: No configuration file found (${file_name})."
-	echo "This script probably won't do anything."
-	return 0
+	# Search in /etc/hibernate
+	if [ -f "$SWSUSP_D/$file_name" ] ; then
+	    file_name="$SWSUSP_D/$file_name"
+	else
+	    echo "WARNING: No configuration file found (${file_name})."
+	    echo "This script probably won't do anything."
+	    return 1
+	fi
     fi
     while true ; do
 	# Doing the read this way allows means we don't require a new-line
@@ -644,7 +675,7 @@ ReadConfigFile() {
 	[ $? -ne 0 ] && [ -z "$option" ] && break
 	[ -z "$option" ] && continue
 	case $option in ""|\#*) continue ;; esac # avoids a function call (big speed hit)
-	ProcessConfigOption $option $params
+	ProcessConfigOption $option $params || break
     done < ${file_name}
     return 0
 }
@@ -667,11 +698,12 @@ AddInbuiltHelp() {
    3: print steps in lots of detail
    4: print out every command executed (uses -x)"
     AddConfigHelp "LogFile <filename>" "If specified, output from the suspend script will also be redirected to this file - useful for debugging purposes."
+    AddConfigHelp "LogTimestamp <boolean>" "If logging to file, will place timestamps on each log entry. This is only recommended if you are trying to optimise suspend/resume time, otherwise it will just slow the process down."
     AddConfigHelp "LogVerbosity N" "Same as Verbosity, but controls what is written to the logfile."
     AddConfigHelp "AlwaysForce <boolean>" "If set to yes, the script will always run as if --force had been passed."
     AddConfigHelp "AlwaysKill <boolean>" "If set to yes, the script will always run as if --kill had been passed."
     AddConfigHelp "Distribution <debian|fedora|mandrake|redhat|gentoo|suse|slackware>" "If specified, tweaks some scriptlets to be more integrated with the given distribution."
-    AddConfigHelp "Include <filename>" "Read configuration directives from the given file."
+    AddConfigHelp "Include <filename>" "Immediately include configuration directives from the given file."
     AddConfigHelp "XDisplay <display location>" "Specifies where scriptlets that use the X server should find one. (Default: :0)"
 }
 
@@ -773,6 +805,13 @@ DISABLE_HELP=1
 EnsureHaveRoot
 LoadScriptlets
 ReadConfigFile "${CONFIG_FILE}"
+# Verify we actually got a suspend method we could use.
+if [ -z "$HIBERNATE_SUSPEND_METHOD" ] ; then
+    echo "$EXE: No suitable suspend methods were found on your machine."
+    echo "$EXE: You need to install a kernel with support for suspending to"
+    echo "$EXE: disk or RAM and reboot, then try again."
+    exit 1
+fi
 ParseOptions "$@"
 
 # Set a logfile if we need one.
@@ -815,4 +854,4 @@ echo "Resumed at "`date` | $LOGPIPE > /dev/null
 
 exit $EXIT_CODE
 
-# $Id: hibernate.sh 975 2005-10-02 05:54:27Z bernard $
+# $Id: hibernate.sh 1027 2006-06-05 11:20:36Z bernard $
